@@ -98,7 +98,7 @@ class Response(object):
         sdata, msg_data = self._decode(fmt, keys, msg_data)
         self.data.update(sdata)
         if msg_data:
-            self.data['additionnal'] = msg_data
+            self.data['additional'] = msg_data
 
         # reformat output, TODO: do it live
         self._format(self.data)
@@ -157,8 +157,67 @@ class R8000(Response):
                         5: ('Stack already started'
                             ' (no new configuration accepted)'),
                         }
+        ZCL_status_codes = {0: 'E_ZCL_SUCCESS',
+                            1: 'E_ZCL_FAIL',
+                            2: 'E_ZCL_ERR_PARAMETER_NULL',
+                            3: 'E_ZCL_ERR_PARAMETER_RANGE',
+                            4: 'E_ZCL_ERR_HEAP_FAIL',
+                            5: 'E_ZCL_ERR_EP_RANGE',
+                            6: 'E_ZCL_ERR_EP_UNKNOWN',
+                            7: 'E_ZCL_ERR_SECURITY_RANGE',
+                            8: 'E_ZCL_ERR_CLUSTER_0',
+                            9: 'E_ZCL_ERR_CLUSTER_NULL',
+                            10: 'E_ZCL_ERR_CLUSTER_NOT_FOUND',
+                            11: 'E_ZCL_ERR_CLUSTER_ID_RANGE',
+                            12: 'E_ZCL_ERR_ATTRIBUTES_NULL',
+                            13: 'E_ZCL_ERR_ATTRIBUTES_0',
+                            14: 'E_ZCL_ERR_ATTRIBUTE_WO',
+                            15: 'E_ZCL_ERR_ATTRIBUTE_RO',
+                            16: 'E_ZCL_ERR_ATTRIBUTES_ACCESS',
+                            17: 'E_ZCL_ERR_ATTRIBUTE_TYPE_UNSUPPORTED',
+                            18: 'E_ZCL_ERR_ATTRIBUTE_NOT_FOUND',
+                            19: 'E_ZCL_ERR_CALLBACK_NULL',
+                            20: 'E_ZCL_ERR_ZBUFFER_FAIL',
+                            21: 'E_ZCL_ERR_ZTRANSMIT_FAIL',
+                            22: 'E_ZCL_ERR_CLIENT_SERVER_STATUS',
+                            23: 'E_ZCL_ERR_TIMER_RESOURCE',
+                            24: 'E_ZCL_ERR_ATTRIBUTE_IS_CLIENT',
+                            25: 'E_ZCL_ERR_ATTRIBUTE_IS_SERVER',
+                            26: 'E_ZCL_ERR_ATTRIBUTE_RANGE',
+                            27: 'E_ZCL_ERR_ATTRIBUTE_MISMATCH',
+                            28: 'E_ZCL_ERR_KEY_ESTABLISHMENT_MORE_THAN_ONE_CLUSTER',
+                            29: 'E_ZCL_ERR_INSUFFICIENT_SPACE',
+                            30: 'E_ZCL_ERR_NO_REPORTABLE_CHANGE',
+                            31: 'E_ZCL_ERR_NO_REPORT_ENTRIES',
+                            32: 'E_ZCL_ERR_ATTRIBUTE_NOT_REPORTABLE',
+                            33: 'E_ZCL_ERR_ATTRIBUTE_ID_ORDER',
+                            34: 'E_ZCL_ERR_MALFORMED_MESSAGE',
+                            35: 'E_ZCL_ERR_MANUFACTURER_SPECIFIC',
+                            36: 'E_ZCL_ERR_PROFILE_ID',
+                            37: 'E_ZCL_ERR_INVALID_VALUE',
+                            38: 'E_ZCL_ERR_CERT_NOT_FOUND',
+                            39: 'E_ZCL_ERR_CUSTOM_DATA_NULL',
+                            40: 'E_ZCL_ERR_TIME_NOT_SYNCHRONISED',
+                            41: 'E_ZCL_ERR_SIGNATURE_VERIFY_FAILED',
+                            42: 'E_ZCL_ERR_ZRECEIVE_FAIL',
+                            43: 'E_ZCL_ERR_KEY_ESTABLISHMENT_END_POINT_NOT_FOUND',
+                            44: 'E_ZCL_ERR_KEY_ESTABLISHMENT_CLUSTER_ENTRY_NOT_FOUND',
+                            45: 'E_ZCL_ERR_KEY_ESTABLISHMENT_CALLBACK_ERROR',
+                            46: 'E_ZCL_ERR_SECURITY_INSUFFICIENT_FOR_CLUSTER',
+                            47: 'E_ZCL_ERR_CUSTOM_COMMAND_HANDLER_NULL_OR_RETURNED_ERROR',
+                            48: 'E_ZCL_ERR_INVALID_IMAGE_SIZE',
+                            49: 'E_ZCL_ERR_INVALID_IMAGE_VERSION',
+                            50: 'E_ZCL_READ_ATTR_REQ_NOT_FINISHED',
+                            51: 'E_ZCL_DENY_ATTRIBUTE_ACCESS',
+                            52: 'E_ZCL_ERR_SECURITY_FAIL',
+                            53: 'E_ZCL_ERR_CLUSTER_COMMAND_NOT_FOUND',
+                            54: 'E_ZCL_ERR_SCENE_NOT_FOUND',
+                            55: 'E_ZCL_RESTORE_DEFAULT_REPORT_CONFIGURATION',
+                            56: 'E_ZCL_ERR_ENUM_END',
+                            }
         return status_codes.get(self.data.get('status'),
-                                'Failed (ZigBee event codes) {:02x}'.format(self.data.get('status')))
+                                ZCL_status_codes.get(self.data.get('status'),
+                                'Failed (ZigBee event codes) 0x{:02x}'.format(self.data.get('status'))))
 
 
 @register_response
@@ -187,35 +246,35 @@ class R8002(Response):
 
     def decode(self):
         Response.decode(self)
-        additionnal = self.data.pop('additionnal')
-        source_address_mode = struct.unpack('!B', additionnal[:1])[0]
+        additional = self.data.pop('additional')
+        source_address_mode = struct.unpack('!B', additional[:1])[0]
         self.data['source_address_mode'] = source_address_mode
-        additionnal = additionnal[1:]
+        additional = additional[1:]
         if source_address_mode == 3:
-            source_address = struct.unpack('!Q', additionnal[:8])[0]
+            source_address = struct.unpack('!Q', additional[:8])[0]
             source_address = '{:016x}'.format(source_address)
-            additionnal = additionnal[8:]
+            additional = additional[8:]
         else:
-            source_address = struct.unpack('!H', additionnal[:2])[0]
+            source_address = struct.unpack('!H', additional[:2])[0]
             source_address = '{:04x}'.format(source_address)
-            additionnal = additionnal[2:]
+            additional = additional[2:]
         self.data['source_address'] = source_address
 
-        dst_address_mode = struct.unpack('!B', additionnal[:1])[0]
+        dst_address_mode = struct.unpack('!B', additional[:1])[0]
         self.data['dst_address_mode'] = dst_address_mode
-        additionnal = additionnal[1:]
+        additional = additional[1:]
         if dst_address_mode == 3:
-            dst_address = struct.unpack('!Q', additionnal[:8])[0]
+            dst_address = struct.unpack('!Q', additional[:8])[0]
             dst_address = '{:016x}'.format(dst_address)
-            additionnal = additionnal[8:]
+            additional = additional[8:]
         else:
-            dst_address = struct.unpack('!H', additionnal[:2])[0]
+            dst_address = struct.unpack('!H', additional[:2])[0]
             dst_address = '{:04x}'.format(dst_address)
-            additionnal = additionnal[2:]
+            additional = additional[2:]
         self.data['dst_address'] = dst_address
-#         payload_size = struct.unpack('!B', additionnal[:1])[0]
+#         payload_size = struct.unpack('!B', additional[:1])[0]
 #         self.data['payload_size'] = payload_size
-        self.data['payload'] = additionnal
+        self.data['payload'] = additional
 
 
 @register_response
@@ -340,7 +399,7 @@ class R8024(Response):
     def decode(self):
         Response.decode(self)
         if self.data['status'] < 2:
-            data = self.data.pop('additionnal')
+            data = self.data.pop('additional')
             data = struct.unpack('!HQB', data)
             self.data['addr'] = data[0]
             self.data['ieee'] = data[1]
@@ -378,7 +437,6 @@ class R8030(Response):
                      ('status', 'B'),
                      ('address_mode', 'B'),
                      ('addr', 'H'),
-                     ('cluster', 'H'),
                      ])
 
     def decode(self):
@@ -386,7 +444,6 @@ class R8030(Response):
             self.s = self.s.copy()
             del self.s['address_mode']
             del self.s['addr']
-            del self.s['cluster']
         Response.decode(self)
 
 
@@ -591,12 +648,12 @@ class R804A(Response):
 
     def decode(self):
         Response.decode(self)
-        additionnal = self.data.pop('additionnal')
-        if self.data['channel_count'] == len(additionnal):
-            channels = struct.unpack('!{}B'.format(self.data['channel_count']), additionnal)
+        additional = self.data.pop('additional')
+        if self.data['channel_count'] == len(additional):
+            channels = struct.unpack('!{}B'.format(self.data['channel_count']), additional)
             self.data['channels'] = [OrderedDict([('channel', c)]) for c in channels]
         else:
-            channels = struct.unpack('!{}BH'.format(self.data['channel_count']), additionnal)
+            channels = struct.unpack('!{}BH'.format(self.data['channel_count']), additional)
             self.data['channels'] = [{'channel': c} for c in channels[:-1]]
             self.data['addr'] = channels[-1]
         self._format(self.data)
@@ -656,16 +713,20 @@ class R804E(Response):
 
     def decode(self):
         Response.decode(self)
-        additionnal = self.data.pop('additionnal')
+        additional = self.data.pop('additional')
         neighbours = []
         for i in range(self.data['count']):
-            neighbour, additionnal = self._decode('!HQQBBB',
-                                                  ['addr', 'extended_panid', 'ieee', 'depth', 'lqi', 'bit_field'],
-                                                  additionnal)
+            if len(additional) < 3:
+                break
+            neighbour, additional = self._decode('!HQQBBB',
+                                                 ['addr', 'extended_panid', 'ieee', 'depth', 'lqi', 'bit_field'],
+                                                 additional)
             neighbours.append(neighbour)
         self.data['neighbours'] = neighbours
-        if additionnal:
-            self.data['addr'] = struct.unpack('!H', additionnal)[0]
+        if len(additional) >= 2:
+            self.data['addr'] = struct.unpack('!H', additional)[0]
+        if 'addr' not in self.data:  # firmware < 3.1a
+            self.data['addr'] = 0xffff
         self._format(self.data)
 # Bit map of attributes Described below: uint8_t
 # {bit 0-1 Device Type
@@ -732,8 +793,8 @@ class R8062(Response):
     def decode(self):
         try:
             Response.decode(self)
-            additionnal = self.data.pop('additionnal')
-            d = struct.unpack('!{}HH'.format(self.data['group_count']), additionnal)
+            additional = self.data.pop('additional')
+            d = struct.unpack('!{}HH'.format(self.data['group_count']), additional)
             self.data['groups'] = [{'group': gaddr} for gaddr in d[:-1]]
             self.data['addr'] = d[-1]
             self._format(self.data)
@@ -905,8 +966,8 @@ class R80A6(Response):
     def decode(self):
         try:
             Response.decode(self)
-            additionnal = self.data.pop('additionnal')
-            d = struct.unpack('!{}BH'.format(self.data['scene_count']), additionnal)
+            additional = self.data.pop('additional')
+            d = struct.unpack('!{}BH'.format(self.data['scene_count']), additional)
             self.data['scenes'] = [{'scene': gaddr} for gaddr in d[:-1]]
             self.data['addr'] = d[-1]
             self._format(self.data, ['addr'])
@@ -1151,9 +1212,9 @@ class R8702(Response):
             self.s['dst_address'] = 'H'
             self.s['sequence'] = 'B'
         Response.decode(self)
-        if 'additionnal' in self.data:
-            additionnal = self.data.pop('additionnal')
-            self.data['dst_address'], self.data['sequence'] = struct.unpack('!QB', additionnal)
+        if 'additional' in self.data:
+            additional = self.data.pop('additional')
+            self.data['dst_address'], self.data['sequence'] = struct.unpack('!QB', additional)
             self.data['dst_address'] = '{:016x}'.format(self.data['dst_address'])
             if self.data['dst_address_mode'] == 2:
                 self.data['dst_address'] = self.data['dst_address'][:4]
